<?php

/**

 * Pimcore

 *

 * LICENSE

 *

 * This source file is subject to the new BSD license that is bundled

 * with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://www.pimcore.org/license

 *

 * @category   Pimcore

 * @package    Object|Class

 * @copyright  Copyright (c) 2009-2014 pimcore GmbH (http://www.pimcore.org)

 * @license    http://www.pimcore.org/license     New BSD License

 */



namespace Pimcore\Model\Object\ClassDefinition\Data\Extension;



use Pimcore\Model;



trait Text {



    /**

     * Checks if data is valid for current data field

     *

     * @param mixed $data

     * @param boolean $omitMandatoryCheck

     * @throws \Exception

     */

    public function checkValidity($data, $omitMandatoryCheck = false)

    {

        if (!$omitMandatoryCheck && $this->getMandatory() && $this->isEmpty($data)) {

            throw new \Exception("Empty mandatory field [ " . $this->getName() . " ]");

        }

    }



    /**

     * @param $data

     * @return bool

     */

    public function isEmpty($data) {

        return (strlen($data) < 1);

    }



    /** True if change is allowed in edit mode.

     * @return bool

     */

    public function isDiffChangeAllowed() {

        return true;

    }



    /**

     * @see Object\ClassDefinition\Data::getVersionPreview

     * @param string $data

     * @return string

     */

    public function getVersionPreview($data) {

        // remove all <script> tags, to prevent XSS in the version preview

        // this should normally be filtered in the project specific controllers/action (/website folder) but just to be sure

        $data = preg_replace('#<script(.*?)>(.*?)</script>#is', '', $data);

        return $data;

    }

}