<?php 

/**

 * Pimcore

 *

 * LICENSE

 *

 * This source file is subject to the new BSD license that is bundled

 * with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://www.pimcore.org/license

 *

 * @category   Pimcore

 * @package    Object|Class

 * @copyright  Copyright (c) 2009-2014 pimcore GmbH (http://www.pimcore.org)

 * @license    http://www.pimcore.org/license     New BSD License

 */



namespace Pimcore\Model\Object\ClassDefinition\Data;



use Pimcore\Model;

use Pimcore\Model\Object;



class Select extends Model\Object\ClassDefinition\Data {



    /**

     * Static type of this element

     *

     * @var string

     */

    public $fieldtype = "select";



    /**

     * Available options to select

     *

     * @var array

     */

    public $options;

    

    /**

     * @var integer

     */

    public $width;    

    

    /**

     * Type for the column to query

     *

     * @var string

     */

    public $queryColumnType = "varchar(255)";



    /**

     * Type for the column

     *

     * @var string

     */

    public $columnType = "varchar(255)";



    /**

     * Type for the generated phpdoc

     *

     * @var string

     */

    public $phpdocType = "string";



    /**

     * @return array

     */

    public function getOptions() {

        return $this->options;

    }



    /**

     * @param array $options

     * @return void

     */

    public function setOptions($options) {

        $this->options = $options;

        return $this;

    }

    

    /**

     * @return integer

     */

    public function getWidth() {

        return $this->width;

    }



    /**

     * @param $width

     * @return $this

     */

    public function setWidth($width) {

        $this->width = $this->getAsIntegerCast($width);

        return $this;

    }



    /**

     * @see Object\ClassDefinition\Data::getDataForResource

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForResource($data, $object = null) {

        return $data;

    }



    /**

     * @see Object\ClassDefinition\Data::getDataFromResource

     * @param string $data

     * @return string

     */

    public function getDataFromResource($data) {

        return $data;

    }



    /**

     * @see Object\ClassDefinition\Data::getDataForQueryResource

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForQueryResource($data, $object = null) {

        return $data;

    }





    /**

     * @see Object\ClassDefinition\Data::getDataForEditmode

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForEditmode($data, $object = null) {

        return $this->getDataForResource($data, $object);

    }



    /**

     * @see Model\Object\ClassDefinition\Data::getDataFromEditmode

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataFromEditmode($data, $object = null) {

        return $this->getDataFromResource($data);

    }



    /**

     * @see Object\ClassDefinition\Data::getVersionPreview

     * @param string $data

     * @return string

     */

    public function getVersionPreview($data) {

        return $data;

    }



    /** True if change is allowed in edit mode.

     * @return bool

     */

    public function isDiffChangeAllowed() {

        return true;

    }



    /** See parent class.

     * @param mixed $data

     * @param null $object

     * @return array|null

     */

    public function getDiffDataForEditMode($data, $object = null) {

        $result = array();



        $diffdata = array();

        $diffdata["data"] = $data;

        $diffdata["disabled"] = false;

        $diffdata["field"] = $this->getName();

        $diffdata["key"] = $this->getName();

        $diffdata["type"] = $this->fieldtype;



        $value = "";

        foreach ($this->options as $option) {

            if ($option["value"] == $data) {

                $value = $option["key"];

                break;

            }

        }



        $diffdata["value"] = $value;

        $diffdata["title"] = !empty($this->title) ? $this->title : $this->name;



        $result[] = $diffdata;



        return $result;

    }



    /**

     * Checks if data is valid for current data field

     *

     * @param mixed $data

     * @param boolean $omitMandatoryCheck

     * @throws \Exception

     */

    public function checkValidity($data, $omitMandatoryCheck = false) {

        if (!$omitMandatoryCheck && $this->getMandatory() && $this->isEmpty($data)) {

            throw new \Exception("Empty mandatory field [ " . $this->getName() . " ]");

        }

    }



    /**

     * @param $data

     * @return bool

     */

    public function isEmpty($data) {

        return (strlen($data) < 1);

    }



    /**

     * @param Object\ClassDefinition\Data $masterDefinition

     */

    public function synchronizeWithMasterDefinition(Object\ClassDefinition\Data $masterDefinition) {

        $this->options = $masterDefinition->options;

    }



}

