<?php 

/**

 * Pimcore

 *

 * LICENSE

 *

 * This source file is subject to the new BSD license that is bundled

 * with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://www.pimcore.org/license

 *

 * @category   Pimcore

 * @package    Object|Class

 * @copyright  Copyright (c) 2009-2014 pimcore GmbH (http://www.pimcore.org)

 * @license    http://www.pimcore.org/license     New BSD License

 */



namespace Pimcore\Model\Object\ClassDefinition\Data;



use Pimcore\Model;



class Textarea extends Model\Object\ClassDefinition\Data {



    use Model\Object\ClassDefinition\Data\Extension\Text;



    /**

     * Static type of this element

     *

     * @var string

     */

    public $fieldtype = "textarea";



    /**

     * @var integer

     */

    public $width;



    /**

     * @var integer

     */

    public $height;



    /**

     * Type for the column to query

     *

     * @var string

     */

    public $queryColumnType = "longtext";



    /**

     * Type for the column

     *

     * @var string

     */

    public $columnType = "longtext";



    /**

     * Type for the generated phpdoc

     *

     * @var string

     */

    public $phpdocType = "string";



    /**

     * @return integer

     */

    public function getWidth() {

        return $this->width;

    }



    /**

     * @return integer

     */

    public function getHeight() {

        return $this->height;

    }



    /**

     * @param integer $width

     * @return void

     */

    public function setWidth($width) {

        $this->width = $this->getAsIntegerCast($width);

        return $this;

    }



    /**

     * @param integer $height

     * @return void

     */

    public function setHeight($height) {

        $this->height = $this->getAsIntegerCast($height);

        return $this;

    }





    /**

     * @see Object\ClassDefinition\Data::getDataForResource

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForResource($data, $object = null) {

        return $data;

    }



    /**

     * @see Object\ClassDefinition\Data::getDataFromResource

     * @param string $data

     * @return string

     */

    public function getDataFromResource($data) {

        return $data;

    }



    /**

     * @see Object\ClassDefinition\Data::getDataForQueryResource

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForQueryResource($data, $object = null) {

        return $data;

    }





    /**

     * @see Object\ClassDefinition\Data::getDataForEditmode

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataForEditmode($data, $object = null) {

        return $this->getDataForResource($data, $object);

    }



    /**

     * @see Model\Object\ClassDefinition\Data::getDataFromEditmode

     * @param string $data

     * @param null|Model\Object\AbstractObject $object

     * @return string

     */

    public function getDataFromEditmode($data, $object = null) {

        return $data;

    }



    /** Generates a pretty version preview (similar to getVersionPreview) can be either html or

     * a image URL. See the ObjectMerger plugin documentation for details

     * @param $data

     * @param null $object

     * @return array|string

     */

    public function getDiffVersionPreview($data, $object = null) {

        if ($data) {

            $value = array();

            $data = str_replace("\r\n", "<br>", $data);

            $data = str_replace("\n", "<br>", $data);

            $data = str_replace("\r", "<br>", $data);



            $value["html"] = $data;

            $value["type"] = "html";

            return $value;

        } else {

            return "";

        }

    }

}

