<?php

/**

 * Pimcore

 *

 * LICENSE

 *

 * This source file is subject to the new BSD license that is bundled

 * with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://www.pimcore.org/license

 *

 * @category   Pimcore

 * @package    Tool

 * @copyright  Copyright (c) 2009-2014 pimcore GmbH (http://www.pimcore.org)

 * @license    http://www.pimcore.org/license     New BSD License

 */



namespace Pimcore\Model\Tool\Qrcode;



use Pimcore\Model;



class Config {



    /**

     * @var string

     */

    public $name = "";



    /**

     * @var string

     */

    public $description = "";



    /**

     * @var string

     */

    public $url = "";



    /**

     * @var

     */

    public $foreColor = "";



    /**

     * @var string

     */

    public $backgroundColor = "";



    /**

     * @var bool

     */

    public $googleAnalytics = true;



    /**

     * @param $name

     * @return Config

     * @throws \Exception

     */

    public static function getByName ($name) {

        $code = new self();

        $code->setName($name);

        if(!$code->load()) {

            throw new \Exception("qr-code definition : " . $name . " does not exist");

        }



        return $code;

    }



    /**

     * @static

     * @return string

     */

    public static function getWorkingDir () {

        $dir = PIMCORE_CONFIGURATION_DIRECTORY . "/qrcodes";

        if(!is_dir($dir)) {

            \Pimcore\File::mkdir($dir);

        }



        return $dir;

    }





    /**

     * @return void

     */

    public function save () {



        $arrayConfig = object2array($this);



        $config = new \Zend_Config($arrayConfig);

        $writer = new \Zend_Config_Writer_Xml(array(

            "config" => $config,

            "filename" => $this->getConfigFile()

        ));

        $writer->write();



        return true;

    }



    /**

     * @return void

     */

    public function load () {



        $configXml = new \Zend_Config_Xml($this->getConfigFile());

        $configArray = $configXml->toArray();



        foreach ($configArray as $key => $value) {

            $setter = "set" . ucfirst($key);

            if(method_exists($this, $setter)) {

                $this->$setter($value);

            }

        }



        return true;

    }



    /**

     * @return void

     */

    public function delete() {

        if(is_file($this->getConfigFile())) {

            unlink($this->getConfigFile());

        }

    }



    /**

     * @return string

     */

    protected function getConfigFile () {

        return self::getWorkingDir() . "/" . $this->getName() . ".xml";

    }



    /**

     * @param $description

     * @return $this

     */

    public function setDescription($description)

    {

        $this->description = $description;

        return $this;

    }



    /**

     * @return string

     */

    public function getDescription()

    {

        return $this->description;

    }



    /**

     * @param $name

     * @return $this

     */

    public function setName($name)

    {

        $this->name = $name;

        return $this;

    }



    /**

     * @return string

     */

    public function getName()

    {

        return $this->name;

    }



    /**

     * @param $url

     * @return $this

     */

    public function setUrl($url)

    {

        $this->url = $url;

        return $this;

    }



    /**

     * @return string

     */

    public function getUrl()

    {

        return $this->url;

    }



    /**

     * @param $backgroundColor

     * @return $this

     */

    public function setBackgroundColor($backgroundColor)

    {

        $this->backgroundColor = $backgroundColor;

        return $this;

    }



    /**

     * @return string

     */

    public function getBackgroundColor()

    {

        return $this->backgroundColor;

    }



    /**

     * @param $foreColor

     * @return $this

     */

    public function setForeColor($foreColor)

    {

        $this->foreColor = $foreColor;

        return $this;

    }



    /**

     * @return string

     */

    public function getForeColor()

    {

        return $this->foreColor;

    }



    /**

     * @param $googleAnalytics

     * @return $this

     */

    public function setGoogleAnalytics($googleAnalytics)

    {

        $this->googleAnalytics = (bool) $googleAnalytics;

        return $this;

    }



    /**

     * @return boolean

     */

    public function getGoogleAnalytics()

    {

        return $this->googleAnalytics;

    }

}