/**

 * Pimcore

 *

 * LICENSE

 *

 * This source file is subject to the new BSD license that is bundled

 * with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://www.pimcore.org/license

 *

 * @copyright  Copyright (c) 2009-2014 pimcore GmbH (http://www.pimcore.org)

 * @license    http://www.pimcore.org/license     New BSD License

 */



pimcore.registerNS("pimcore.settings.thumbnail.panel");

pimcore.settings.thumbnail.panel = Class.create({



    initialize: function () {



        this.getTabPanel();

    },



    activate: function () {

        var tabPanel = Ext.getCmp("pimcore_panel_tabs");

        tabPanel.activate("pimcore_thumbnails");

    },



    getTabPanel: function () {



        if (!this.panel) {

            this.panel = new Ext.Panel({

                id: "pimcore_thumbnails",

                title: t("thumbnails"),

                iconCls: "pimcore_icon_thumbnails",

                border: false,

                layout: "border",

                closable:true,

                items: [this.getTree(), this.getEditPanel()]

            });



            var tabPanel = Ext.getCmp("pimcore_panel_tabs");

            tabPanel.add(this.panel);

            tabPanel.activate("pimcore_thumbnails");



            this.panel.on("destroy", function () {

                pimcore.globalmanager.remove("thumbnails");

            }.bind(this));



            pimcore.layout.refresh();

        }



        return this.panel;

    },

    

    getTree: function () {

        if (!this.tree) {

            this.tree = new Ext.tree.TreePanel({

                id: "pimcore_panel_thumbnail_tree",

                region: "west",

                useArrows:true,

                autoScroll:true,

                animate:true,

                containerScroll: true,

                border: true,

                width: 200,

                split: true,

                root: {

                    nodeType: 'async',

                    id: '0'

                },

                loader: new Ext.tree.TreeLoader({

                    dataUrl: '/admin/settings/thumbnail-tree',

                    requestMethod: "GET",

                    baseAttrs: {

                        listeners: this.getTreeNodeListeners(),

                        reference: this,

                        allowDrop: false,

                        allowChildren: false,

                        isTarget: false,

                        iconCls: "pimcore_icon_thumbnails",

                        leaf: true

                    }

                }),

                rootVisible: false,

                tbar: {

                    items: [

                        {

                            text: t("add_thumbnail"),

                            iconCls: "pimcore_icon_add",

                            handler: this.addField.bind(this)

                        }

                    ]

                }

            });



            this.tree.on("render", function () {

                this.getRootNode().expand();

            });

        }



        return this.tree;

    },



    getEditPanel: function () {

        if (!this.editPanel) {

            this.editPanel = new Ext.TabPanel({

                region: "center"

            });

        }



        return this.editPanel;

    },



    getTreeNodeListeners: function () {

        var treeNodeListeners = {

            'click' : this.onTreeNodeClick.bind(this),

            "contextmenu": this.onTreeNodeContextmenu

        };



        return treeNodeListeners;

    },



    onTreeNodeClick: function (node) {

        this.openThumbnail(node.id);

    },



    openThumbnail: function(id) {



        var existingPanel = Ext.getCmp("pimcore_thumbnail_panel_" + id);

        if(existingPanel) {

            this.editPanel.activate(existingPanel);

            return;

        }



        Ext.Ajax.request({

            url: "/admin/settings/thumbnail-get",

            params: {

                name: id

            },

            success: function (response) {

                var data = Ext.decode(response.responseText);



                var fieldPanel = new pimcore.settings.thumbnail.item(data, this);

                pimcore.layout.refresh();

            }.bind(this)

        });

    },



    onTreeNodeContextmenu: function () {

        this.select();



        var menu = new Ext.menu.Menu();

        menu.add(new Ext.menu.Item({

            text: t('delete'),

            iconCls: "pimcore_icon_delete",

            handler: this.attributes.reference.deleteField.bind(this)

        }));



        menu.show(this.ui.getAnchor());

    },



    addField: function () {

        Ext.MessageBox.prompt(t('add_thumbnail'), t('enter_the_name_of_the_new_thumbnail'),

                                this.addFieldComplete.bind(this), null, null, "");

    },



    addFieldComplete: function (button, value, object) {



        var regresult = value.match(/[a-zA-Z0-9_\-]+/);



        if (button == "ok" && value.length > 2 && regresult == value) {



            var thumbnails = this.tree.getRootNode().childNodes;

            for (var i = 0; i < thumbnails.length; i++) {

                if (thumbnails[i].text == value) {

                    Ext.MessageBox.alert(t('add_thumbnail'),

                        t('the_key_is_already_in_use_in_this_level_please_choose_an_other_key'));

                    return;

                }

            }



            Ext.Ajax.request({

                url: "/admin/settings/thumbnail-add",

                params: {

                    name: value

                },

                success: function (response) {

                    var data = Ext.decode(response.responseText);



                    this.tree.getRootNode().reload();



                    if(!data || !data.success) {

                        Ext.Msg.alert(t('add_thumbnail'), t('problem_creating_new_thumbnail'));

                    } else {

                        this.openThumbnail(data.id);

                    }

                }.bind(this)

            });

        }

        else if (button == "cancel") {

            return;

        }

        else {

            Ext.Msg.alert(t('add_thumbnail'), t('problem_creating_new_thumbnail'));

        }

    },



    deleteField: function () {

        Ext.Ajax.request({

            url: "/admin/settings/thumbnail-delete",

            params: {

                name: this.id

            }

        });



        this.attributes.reference.getEditPanel().removeAll();

        this.remove();

    }

});



